﻿using Newtonsoft.Json;
using rest_api_test.Forms;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace rest_api_test
{

    public class QueryObject
    {
        public bool IsAnd { get; set; }
        public string FilterName { get; set; }
        public string Operator { get; set; }
        public string Value { get; set; }
    }

    class Program
    {
        private static string UserName = Constants.UserName;
        private static string Password = Constants.Password;
        private static string Website = "https://forms.agterra.com";
        private static int WebTimeout = 1000 * 60 * 5; // 5 minutes.

        static void Main(string[] args)
        {
            TestGetMultiple();
            Console.WriteLine("Press any key to exit");
            Console.ReadKey();
        }

        #region testing functions

        private static void TestGetSingle()
        {
            var result = GetSingle<MyFormList>(MyFormList.FormId, 10);
            Console.WriteLine("Received Record.");
        }

        private static void TestGetMultiple()
        {
            List<QueryObject> objs = new List<QueryObject>();
            objs.Add(new QueryObject() { IsAnd = true, FilterName = "decimal1", Operator = ">", Value = "1" });
            objs.Add(new QueryObject() { IsAnd = true, FilterName = "decimal", Operator = "<", Value = "99" });
            var result = GetMultiple<MyFormList>(MyFormList.FormId, JsonConvert.SerializeObject(objs)); // NOTE: WILL RETURN NULL IF NO RECORDS FOUND!
            int numRecords = 0;
            if (result != null && result.My_Form_ID != null)
                numRecords = result.My_Form_ID.Count;
            Console.WriteLine("Received " + numRecords + " Records.");
        }

        // Update
        private static void TestPut()
        {
            var result = GetSingle<MyFormList>(MyFormList.FormId, 10);
            result.My_Form_ID[0].decimal1 = (decimal)123.456;
            var pr = PutRequest(MyFormList.FormId, 10, JsonConvert.SerializeObject(result));
            Console.WriteLine("Record Deleted");
        }

        // Insert
        private static void TestPost()
        {
            var result = GetSingle<MyFormList>(MyFormList.FormId, 10);
            var pr = PostRequest(MyFormList.FormId, JsonConvert.SerializeObject(result));
            Console.WriteLine("Record Inserted.");
        }

        private static void TestDelete()
        {
            DeleteRequest(MyFormList.FormId, 9999);
            Console.WriteLine("Record Deleted.");
        }

        #endregion

        #region Main Functions

        private static T GetSingle<T>(string form, int id)
        {
            string getAddress = new Uri(string.Format("{0}/api/{1}/Get/{2}", Website, form, id)).ToString();
            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(getAddress);
            httpWebRequest.Headers.Add("Authorization", "Basic " + Convert.ToBase64String(System.Text.ASCIIEncoding.ASCII.GetBytes(string.Format("{0}:{1}", UserName, Password))));
            httpWebRequest.Method = "GET";
            httpWebRequest.Timeout = WebTimeout;
            HttpWebResponse httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            StreamReader getSr = new StreamReader(httpWebResponse.GetResponseStream());
            string strResponse = getSr.ReadToEnd();
            try
            {
                return Newtonsoft.Json.JsonConvert.DeserializeObject<T>(strResponse);
            }
            catch (Exception)
            {
                return default(T);
            }
        }

        private static T GetMultiple<T>(string formName, string jsonObj)
        {
            string address = new Uri(string.Format("{0}/api/{1}/Filtered/0", Website, formName)).ToString();
            var httpWebRequest = (HttpWebRequest)WebRequest.Create(address);
            httpWebRequest.Headers.Add("Authorization", "Basic " + Convert.ToBase64String(System.Text.ASCIIEncoding.ASCII.GetBytes(string.Format("{0}:{1}", UserName, Password))));
            httpWebRequest.ContentType = "text/json";
            httpWebRequest.Timeout = WebTimeout;
            httpWebRequest.Method = "POST";

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                streamWriter.Write(jsonObj);
                streamWriter.Flush();
                streamWriter.Close();

                var httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();
                using (var streamReader = new StreamReader(httpWebResponse.GetResponseStream()))
                {
                    var result = streamReader.ReadToEnd();
                    try
                    {
                        return Newtonsoft.Json.JsonConvert.DeserializeObject<T>(result);
                    } 
                    catch(Exception)
                    {
                        return default(T);
                    }
                }
            }
        }

        private static void DeleteRequest(string formName, int id)
        {
            string addressDelete = new Uri(string.Format("{0}/api/{1}/Delete/{2}", Website, formName, id)).ToString();
            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(addressDelete);
            httpWebRequest.Headers.Add("Authorization", "Basic " + Convert.ToBase64String(System.Text.ASCIIEncoding.ASCII.GetBytes(string.Format("{0}:{1}", UserName, Password))));
            httpWebRequest.Method = "GET";
            httpWebRequest.Timeout = WebTimeout;
            HttpWebResponse httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            StreamReader deleteSr = new StreamReader(httpWebResponse.GetResponseStream());
            Console.WriteLine(deleteSr.ReadToEnd());
        }

        private static string PutRequest(string formName, int id, string jsonObject)
        {
            string address = new Uri(string.Format("{0}/api/{1}/Put/{2}", Website, formName, id)).ToString();
            var httpWebRequest = (HttpWebRequest)WebRequest.Create(address);
            httpWebRequest.Headers.Add("Authorization", "Basic " + Convert.ToBase64String(System.Text.ASCIIEncoding.ASCII.GetBytes(string.Format("{0}:{1}", UserName, Password))));
            httpWebRequest.Timeout = WebTimeout;
            httpWebRequest.ContentType = "text/json";
            httpWebRequest.Method = "POST";
            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                streamWriter.Write(jsonObject);
                streamWriter.Flush();
                streamWriter.Close();

                var httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();
                using (var streamReader = new StreamReader(httpWebResponse.GetResponseStream()))
                {
                    return streamReader.ReadToEnd();
                }
            }
        }

        private static string PostRequest(string formName, string jsonObject)
        {
            string address = new Uri(string.Format("{0}/api/{1}/Post/0", Website, formName)).ToString();
            var httpWebRequest = (HttpWebRequest)WebRequest.Create(address);
            httpWebRequest.Headers.Add("Authorization", "Basic " + Convert.ToBase64String(System.Text.ASCIIEncoding.ASCII.GetBytes(string.Format("{0}:{1}", UserName, Password))));
            httpWebRequest.ContentType = "text/json";
            httpWebRequest.Timeout = WebTimeout;
            httpWebRequest.Method = "POST";

            using (var streamWriter = new StreamWriter(httpWebRequest.GetRequestStream()))
            {
                streamWriter.Write(jsonObject);
                streamWriter.Flush();
                streamWriter.Close();

                var httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();
                using (var streamReader = new StreamReader(httpWebResponse.GetResponseStream()))
                {
                    return streamReader.ReadToEnd();
                }
            }
        }

        #endregion
    }
}
